"""
Actions for filtering based on temporal information
"""

from calendar import monthrange
from datetime import date, datetime
from logging import getLogger
from typing import Iterable

from spotify_model import ReleaseDatePrecision, SearchAlbum


def temporal_released_after(albums: Iterable[SearchAlbum], released_after: date) -> Iterable[SearchAlbum]:
    """
    Filter albums to after a specific release date.

    For albums that don't have date-level precision, the release date is treated as the final day within that period;
    thus, albums released in "1981" are effectively released on 1981-12-31, and albums released in "1981-07" are
    treated as "1981-07-31"
    """

    logger = getLogger(__name__)

    for album in albums:
        if album.release_date_precision == ReleaseDatePrecision.YEAR:
            actual_release = datetime.strptime(album.release_date, "%Y")
            effective_release = date(actual_release.year, 12, 31)
        elif album.release_date_precision == ReleaseDatePrecision.MONTH:
            actual_release = datetime.strptime(album.release_date, "%Y-%m")
            final_day = monthrange(actual_release.year, actual_release.month)[1] - 1
            effective_release = date(actual_release.year, actual_release.month, final_day)
        else:
            effective_release = datetime.strptime(album.release_date, "%Y-%m-%d").date()

        if effective_release >= released_after:
            logger.debug(
                "Including album=%s released on date=%s (prior to date=%s)",
                album.name,
                effective_release,
                released_after,
            )
            yield album
        else:
            logger.debug(
                "Skipping album=%s released on date=%s (prior to date=%s)",
                album.name,
                effective_release,
                released_after,
            )
